% LemkeHowson Solves mixed strategies for bimatrix game.
% LemkeHowson2 is designed to find the mix NE for cost matrix which means
% to find the minimum strategies.
%   [p,q,err]=LemkeHowson2(A,B);
% ERR returns an error condition:
%   0: Solution found
%   1: Maximum iterations exceeded

function [p,q] = LemkeHowson2(A,B)
% to gurantee T-A and T-B is positive
T=max([max(max(A)),max(max(B))])*2*ones(size(A));
%err=0;
k0=1;
%MAX=100;
A=T-A;
B=T-B;
[m n]=size(A);
P=[B',eye(n),ones(n,1)];
Q=[eye(m),A,ones(m,1)];

[p q r s] = Tableau(P,Q,k0);
while min(p>=0)*min(q>=0)*min(r>=0)*min(s>=0)==0
    k0=k0+1;
    [p q r s] = Tableau(P,Q,k0);
end
end


function [p q r s] = Tableau(P,Q,k0)
m=size(Q,1);
n=size(P,1);
LP=zeros(1,m);
LQ=zeros(1,n);
for i=1:m
    LP(i)=i;
end
for i=1:n
    LQ(i)=i+m;
end
% initial basis
BP=LQ;
BQ=LP;

% initial labels
m=size(Q,1);
n=size(P,1);
% begine pivot
k=k0;
while 1
    kp=P(:,k)./P(:,m+n+1);
    tmp=0;
    while tmp==0
        [tmp i]=max(kp);
        kp(i)=-inf;
    end
    P=pivot(P,i,k);
    % BP(i) needs to leave the basis of P
    tmp=k;
    k=BP(i);
    BP(i)=tmp;
    %P
    if k==k0
        break;
    end
    kq=Q(:,k)./Q(:,m+n+1);
    tmp=0;
    while tmp==0
        [tmp i]=max(kq);
        kq(i)=-inf;
    end
    Q=pivot(Q,i,k);
    % BQ(i) needs to leave the basis of Q
    tmp=k;
    k=BQ(i);
    BQ(i)=tmp;
    %Q
    if k==k0
        break;
    end
end
p=zeros(m,1);
q=zeros(n,1);
r=zeros(m,1);
s=zeros(n,1);
for i=1:n
    k=BP(i);
    if k<=m
        p(k)=P(i,m+n+1)/P(i,k);
    else
        s(k-m)=P(i,m+n+1)/P(i,k);
    end
end

for i=1:m
    k=BQ(i);
    if k<=m
        r(k)=Q(i,m+n+1)/Q(i,k);
    else
        q(k-m)=Q(i,m+n+1)/Q(i,k);
    end
end
cb=1/sum(p);
ca=1/sum(q);
p=cb*p;
q=ca*q;
r=ca*r;
s=cb*s;
end

function P=pivot(P,i,j)
assert(P(i,j)~=0);
r=size(P,1);
for k=1:r
    if k==i
        continue;
    end
    if P(k,j)==0
        continue;
    end
    P(k,:)=P(k,:)*P(i,j)-P(i,:)*P(k,j);
   % P(k,:)=P(k,:)/gcd2(P(k,:));
end
end

